/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.persistence;

import java.util.Collection;
import java.util.Set;

import org.ow2.bonita.definition.GlobalClassData;
import org.ow2.bonita.definition.PackageClassData;
import org.ow2.bonita.definition.PackageDependency;
import org.ow2.bonita.definition.XpdlProcess;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.pvm.job.Timer;
import org.ow2.bonita.pvm.session.PvmDbSession;
import org.ow2.bonita.runtime.ActivityInstanceKey;
import org.ow2.bonita.runtime.PackageVersion;
import org.ow2.bonita.runtime.ProcessInstanceKey;
import org.ow2.bonita.runtime.ProcessVersion;
import org.ow2.bonita.runtime.TaskRunTime;
import org.ow2.bonita.runtime.XpdlExecution;
import org.ow2.bonita.runtime.XpdlInstance;


public interface RuntimeDbSession extends PvmDbSession {

  XpdlInstance getXpdlInstance(ProcessInstanceUUID instanceUUID);

  Set<XpdlInstance> getXpdlInstances(ProcessDefinitionUUID processId);

  XpdlProcess getXpdlProcess(ProcessDefinitionUUID processUUID);

  Set<TaskRunTime> getInstanceTasks(ProcessInstanceUUID instanceUUID);

  ProcessVersion getLastProcessVersion(String processId, String packageId);
  PackageVersion getLastPackageVersion(String packageId);


  XpdlProcess findLatestProcessById(String subProcessId);

  ProcessInstanceKey getProcessInstanceKey(ProcessDefinitionUUID processUUID);

  ActivityInstanceKey getActivityInstanceKey(ProcessInstanceUUID instanceUUID,
      String activityName);

  Collection<Timer> getExecutionTimers(XpdlExecution xpdlExecution);

  TaskRunTime getTask(TaskUUID taskUUID);

  XpdlExecution getExecutionPointingOnNode(ActivityInstanceUUID activityUUID);

  Set<PackageDependency> getPackageDependencies(String className);
  PackageDependency getPackageDependency(PackageDefinitionUUID packageId);

  PackageClassData getPackageClassData(PackageDefinitionUUID processId);
  GlobalClassData getGlobalClassData(String className);

  Set<Timer> getInstanceTimers(ProcessInstanceUUID instanceUUID);
}


